package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.entity.BaseModel;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.BaseUtils;
import pps.core.space.entity.SpaceCalibrationHistoryView;
import pps.core.space.entity.SpaceCalibrationPeriodEnt;
import pps.core.space.mapper.SpaceCalibrationHistoryViewMapper;
import pps.core.space.mapper.SpaceCalibrationPeriodMapper;
import pps.core.space.service.data.space_calibration_history.QuerySpaceCalibrationHistoryViewInput;
import pps.core.space.service.data.space_calibration_history.QuerySpaceCalibrationHistoryViewOutput;
import pps.core.space.service.data.space_calibration_period.CreateSpaceCalibrationPeriodInput;
import pps.core.space.service.data.space_calibration_period.GetSpaceCalibrationPeriodViewInput;
import pps.core.space.service.data.space_calibration_period.GetSpaceCalibrationPeriodViewOutput;
import pps.core.space.service.data.space_calibration_period.UpdateSpaceCalibrationPeriodInput;
import pps.core.space.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 定期校准模块
 *
 * @author ZWT
 * @date 2023/09/06
 */
@XService
@XText("定期校准模块")
public class SpaceCalibrationService {

    /**
     * 校准周期--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("校准周期--新增")
    public XServiceResult createSpaceCalibrationPeriod(XContext context, CreateSpaceCalibrationPeriodInput input) {
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
//        PpsUserSession session = context.getSession(PpsUserSession.class);
        return XTransactionHelper.begin(context, () -> {
            SpaceCalibrationPeriodMapper mapper = context.getBean(SpaceCalibrationPeriodMapper.class);
            SpaceCalibrationPeriodEnt periodEnt = this.getPeriodEntByParam(mapper, input.getLineId(), input.getOuId());
            if (Objects.nonNull(periodEnt)) {
                return XServiceResult.error(992, "当前线路已设定周期");
            }
            SpaceCalibrationPeriodEnt entity = XCopyUtils.copyNewObject(input, SpaceCalibrationPeriodEnt.class);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.insert(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 校准周期--修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("校准周期--修改")
    public XServiceResult updateSpaceCalibrationPeriod(XContext context, UpdateSpaceCalibrationPeriodInput input) {
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
//        PpsUserSession session = context.getSession(PpsUserSession.class);
        return XTransactionHelper.begin(context, () -> {
            SpaceCalibrationPeriodMapper mapper = context.getBean(SpaceCalibrationPeriodMapper.class);
            SpaceCalibrationPeriodEnt entity = mapper.selectOne(
                    new LambdaQueryWrapper<SpaceCalibrationPeriodEnt>()
                            .eq(BaseModel::getId, input.getId())
            );
            if (Objects.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            BaseUtils.setBaseModelDefault(entity, session);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    /**
     * 校准周期--详情
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XSingleResult}<{@link GetSpaceCalibrationPeriodViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("校准周期--详情")
    public XSingleResult<GetSpaceCalibrationPeriodViewOutput> getSpaceCalibrationPeriodView(XContext context, GetSpaceCalibrationPeriodViewInput input) {
        String lineId = input.getLineId();
        String ouId = input.getOuId();
        SpaceCalibrationPeriodMapper mapper = context.getBean(SpaceCalibrationPeriodMapper.class);
        SpaceCalibrationPeriodEnt entity = this.getPeriodEntByParam(mapper, lineId, ouId);
        GetSpaceCalibrationPeriodViewOutput output = new GetSpaceCalibrationPeriodViewOutput();
        if (Objects.nonNull(entity)) {
            XCopyUtils.copyObject(entity, output);
        }
        output.setLineName(
                ServiceUtil.getPowerLineNameById(context, lineId)
        );
        output.setOuName(
                ServiceUtil.getSysOrgNameById(context, ouId)
        );
        return XSingleResult.success(output);
    }

    /**
     * 校准历史--分页列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySpaceCalibrationHistoryViewOutput}>
     */
    @XApiAnonymous
    @XApiGet
    @XText("校准历史--分页列表")
    public XPageResult<QuerySpaceCalibrationHistoryViewOutput> querySpaceCalibrationHistoryView(XContext context, QuerySpaceCalibrationHistoryViewInput input) {
        SpaceCalibrationHistoryViewMapper mapper = context.getBean(SpaceCalibrationHistoryViewMapper.class);
        SpaceCalibrationHistoryView record = XCopyUtils.copyNewObject(input, SpaceCalibrationHistoryView.class);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SpaceCalibrationHistoryView> list = mapper.selectList(record);
        PageInfo<SpaceCalibrationHistoryView> pageInfo = new PageInfo<>(list);
        List<QuerySpaceCalibrationHistoryViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySpaceCalibrationHistoryViewOutput.class);
        if (CollUtil.isNotEmpty(outputs)) {
            Map<String, String> gridTypeMap = ServiceUtil.getDictMap(context, BusinessConstant.GRID_TYPE);
            Map<String, String> ouMap = ServiceUtil.getOuMapByOuIds(context, outputs.stream()
                    .map(QuerySpaceCalibrationHistoryViewOutput::getOuId)
                    .distinct()
                    .collect(Collectors.toList())
            );
            String lineName = ServiceUtil.getPowerLineNameById(context, input.getLineId());
            outputs.forEach(output -> {
                output.setLineName(lineName);
                output.setOuName(ouMap.get(output.getOuId()));
                output.setGridTypeName(gridTypeMap.get(output.getGridTypeKey()));
            });
        }
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /*-----------------------------------private-----------------------------------*/

    /**
     * 通过参数获取周期
     *
     * @param mapper 映射器
     * @param lineId 线路id
     * @param ouId   ou-id
     * @return {@link SpaceCalibrationPeriodEnt}
     */
    private SpaceCalibrationPeriodEnt getPeriodEntByParam(SpaceCalibrationPeriodMapper mapper, String lineId, String ouId) {
        return mapper.selectOne(
                new LambdaQueryWrapper<SpaceCalibrationPeriodEnt>()
                        .eq(SpaceCalibrationPeriodEnt::getLineId, lineId)
                        .eq(SpaceCalibrationPeriodEnt::getOuId, ouId)
        );
    }
}
